@@ -1,19 +1,22 @@
+from __future__ import annotations
+
 import time
+from typing import Literal
 
 
 class TimerContext:
     """context manager for timing code"""
 
-    def __init__(self):
-        self.start_time: float = None
-        self.end_time: float = None
-        self.elapsed_time: float = None
+    def __init__(self) -> None:
+        self.start_time: float
+        self.end_time: float
+        self.elapsed_time: float
 
-    def __enter__(self):
+    def __enter__(self) -> "TimerContext":
         self.start_time = time.time()
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type, exc_val, exc_tb) -> Literal[False]:
         self.end_time = time.time()
         self.elapsed_time = self.end_time - self.start_time
         return False
@@ -78,6 +81,7 @@ class ProgressEstimator:
         timing_raw: dict[str, float] = self.get_timing_raw(i)
 
         percent_str: str = str(int(timing_raw["percent"] * 100)).ljust(2)
-        iters_str: str = f"{str(i).ljust(self.total_str_len)}/{self.n_total}"
-        timing_str: str
+        # TODO: get_progress_default
+        # iters_str: str = f"{str(i).ljust(self.total_str_len)}/{self.n_total}"
+        # timing_str: str
         return f"{percent_str}% {self.get_pbar(i)}"
