@@ -1,21 +1,19 @@
+"""`StatCounter` class for counting and calculating statistics on numbers
+
+cleaner and more efficient than just using a `Counter` or array"""
+
+from __future__ import annotations
+
 import json
 import math
 from collections import Counter
 from functools import cached_property
 from itertools import chain
-from types import NoneType
-from typing import Callable, Iterable, Optional, Sequence, Union
-
-import numpy as np
-
-# import pdb
+from typing import Callable, Optional, Sequence, Union
 
 
-# TODO: mypy complains that "torch" is not defined
 # _GeneralArray = Union[np.ndarray, "torch.Tensor"]
-_GeneralArray = np.ndarray
-
-GeneralSequence = Union[Sequence, _GeneralArray]
+NumericSequence = Sequence[Union[float, int, "NumericSequence"]]
 
 # pylint: disable=abstract-method
 
@@ -24,34 +22,23 @@ GeneralSequence = Union[Sequence, _GeneralArray]
 
 
 def universal_flatten(
-    arr: GeneralSequence, require_rectangular: bool = True
-) -> GeneralSequence:
+    arr: Union[NumericSequence, float, int], require_rectangular: bool = True
+) -> NumericSequence:
     """flattens any iterable"""
 
     # mypy complains that the sequence has no attribute "flatten"
     if hasattr(arr, "flatten") and callable(arr.flatten):  # type: ignore
         return arr.flatten()  # type: ignore
-    elif not isinstance(arr, Iterable):
-        return arr
-    else:
-        elements_iterable: list[bool] = [isinstance(x, Iterable) for x in arr]
+    elif isinstance(arr, Sequence):
+        elements_iterable: list[bool] = [isinstance(x, Sequence) for x in arr]
         if require_rectangular and (all(elements_iterable) != any(elements_iterable)):
             raise ValueError("arr contains mixed iterable and non-iterable elements")
         if any(elements_iterable):
-            return list(chain.from_iterable(universal_flatten(x) for x in arr))
+            return list(chain.from_iterable(universal_flatten(x) for x in arr))  # type: ignore[misc]
         else:
             return arr
-
-
-def compute_rsquared(actual: np.ndarray, predicted: np.ndarray) -> float:
-    assert actual.shape == predicted.shape
-
-    actual_mean: float = np.mean(actual)
-
-    ss_res: float = np.sum((actual - predicted) ** 2.0)
-    ss_tot: float = np.sum((actual - actual_mean) ** 2.0)
-
-    return 1 - ss_res / ss_tot
+    else:
+        return [arr]
 
 
 # StatCounter
@@ -59,21 +46,27 @@ def compute_rsquared(actual: np.ndarray, predicted: np.ndarray) -> float:
 
 
 class StatCounter(Counter):
-    """counter, but with some stat calculation methods which assume the keys are numbers
+    """`Counter`, but with some stat calculation methods which assume the keys are numerical
 
-    works best when the keys are ints
+    works best when the keys are `int`s
     """
 
     def validate(self) -> bool:
         """validate the counter as being all floats or ints"""
-        return all(isinstance(k, (bool, int, float, NoneType)) for k in self.keys())
+        return all(isinstance(k, (bool, int, float, type(None))) for k in self.keys())
 
     def min(self):
+        "minimum value"
         return min(x for x, v in self.items() if v > 0)
 
     def max(self):
+        "maximum value"
         return max(x for x, v in self.items() if v > 0)
 
+    def total(self):
+        """Sum of the counts"""
+        return sum(self.values())
+
     @cached_property
     def keys_sorted(self) -> list:
         """return the keys"""
@@ -151,6 +144,7 @@ class StatCounter(Counter):
         *,
         extra_percentiles: Optional[list[float]] = None,
     ) -> dict[str, Union[float, int]]:
+        """return a summary of the stats, without the raw data. human readable and small"""
         # common stats that always work
         output: dict = dict(
             total_items=self.total(),
@@ -190,6 +184,18 @@ class StatCounter(Counter):
         *,
         extra_percentiles: Optional[list[float]] = None,
     ) -> dict:
+        """return a json-serializable version of the counter
+
+        includes both the output of `summary` and the raw data:
+
+        ```json
+        {
+            "StatCounter": { <keys, values from raw data> },
+            "summary": self.summary(typecast, extra_percentiles=extra_percentiles),
+        }
+
+        """
+
         return {
             "StatCounter": {
                 typecast(k): v
@@ -199,6 +205,7 @@ class StatCounter(Counter):
         }
 
     def __str__(self) -> str:
+        "summary as json with 2 space indent, good for printing"
         return json.dumps(self.summary(), indent=2)
 
     def __repr__(self) -> str:
@@ -206,6 +213,7 @@ class StatCounter(Counter):
 
     @classmethod
     def load(cls, data: dict) -> "StatCounter":
+        "load from a the output of `StatCounter.serialize`"
         if "StatCounter" in data:
             loadme = data["StatCounter"]
         else:
@@ -216,70 +224,8 @@ class StatCounter(Counter):
     @classmethod
     def from_list_arrays(
         cls,
-        arr: _GeneralArray,
+        arr,
         map_func: Callable = float,
     ) -> "StatCounter":
         """calls `map_func` on each element of `universal_flatten(arr)`"""
         return cls([map_func(x) for x in universal_flatten(arr)])
-
-
-# testing
-# ==================================================
-
-
-def _compute_err(a: float, b: float, /) -> dict[str, float]:
-    return dict(
-        num_a=float(a),
-        num_b=float(b),
-        diff=float(b - a),
-        frac_err=float((b - a) / a),
-    )
-
-
-def _compare_np_custom(arr: np.ndarray) -> dict[str, dict]:
-    counter: StatCounter = StatCounter(arr)
-    return dict(
-        mean=_compute_err(counter.mean(), np.mean(arr)),
-        std=_compute_err(counter.std(), np.std(arr)),
-        min=_compute_err(counter.min(), np.min(arr)),
-        q1=_compute_err(counter.percentile(0.25), np.percentile(arr, 25)),
-        median=_compute_err(counter.median(), np.median(arr)),
-        q3=_compute_err(counter.percentile(0.75), np.percentile(arr, 75)),
-        max=_compute_err(counter.max(), np.max(arr)),
-    )
-
-
-def _test_statscounter():
-    arrs: list[np.ndarray] = [
-        np.array([0, 1]),
-        np.array([1, 2]),
-        np.random.randint(0, 10, size=100),
-        np.random.randint(-5, 15, size=10),
-        np.array([-5, -4, -1, 1, 1, 3, 3, 5, 11, 12]),
-        np.random.randint(-5, 15, size=10000),
-        np.random.randint(0, 100, size=100),
-        np.random.randint(0, 1000, size=10000),
-    ]
-
-    for i, j in np.random.randint(1, 100, size=(50, 2)):
-        if i > j:
-            i, j = j, i
-
-        arrs.append(np.random.randint(i, j, size=1000))
-
-    import json
-
-    for a in arrs:
-        r = _compare_np_custom(a)
-
-        if any(x["diff"] > 0.00000000001 for x in r.values()):
-            print("!!!!!!!!!!!!!!!!!!!!")
-            print(f"errs for rantint array: {a.shape = } {np.min(a) =} {np.max(a) =}")
-            print(json.dumps(r, indent=2))
-        # s = StatCounter(a)
-        # print(s.total(), s)
-        # print(sorted(list(s.elements())))
-
-
-# if __name__ == '__main__':
-#     _test_statscounter()
